/* -----------------------------------------

   Create deltaC x-tables for all available values of alphas

   ----------------------------------------- */
#include <iomanip>
#include <iostream>
#include <fstream>
#include <string>
#include <cmath>
#include <ctime>
#include <cstdlib>
#include <sys/time.h> 
#include <getopt.h> 

#include "hell.hh"


using namespace std;

double Qofalphas(double as, double as0, double Q0, int nf) {
  double b0 = (33 - 2*nf)/12./M_PI;
  double b1 = (153-19*nf)/24./M_PI/M_PI/b0;
  return Q0 * exp(1./2./b0 * (1./as - 1./as0 - b1*log((b1 + 1./as)/(b1 + 1./as0))));
}
const double mZ = 91.1876;
const double asZ[4] = {0.10466, 0.11227, 0.118, 0.118};
double Qofalphas(double as, int nf) {
  return Qofalphas(as, asZ[nf-3], mZ, nf);
}


string sas(double as) {
  ostringstream os;
  int ias = int(1000*as+0.1);
  if(1000*as-ias>0.5) ias += 1;
  if     (ias<10  ) os << "000" << ias;
  else if(ias<100 ) os << "00"  << ias;
  else if(ias<1000) os << "0"   << ias;
  else os << ias;
  return os.str();
}


void print_usage() {
  cout << endl
       << "Usage:" << endl
       << " -a  --alphas <as>      set alpha_s" << endl
       << " -n  --nf <nf>          set the number of active flavours" << endl
       << " -L  --LLp              use LLp (default is NLL)" << endl
       << endl;
  exit(0);
  return;
}

void read_arguments(int argc, char* argv[], double &as, int &nf, double &muFrat, bool &useLLp) {
  const char* const short_options = "ha:n:F:L";
  const struct option long_options[] = { { "help", 0, NULL, 'h' },
					 { "alphas", 1, NULL, 'a' },
					 { "nf", 1, NULL, 'n' },
					 { "muF", 1, NULL, 'F' },
					 { "LLp", 0, NULL, 'L' },
					 { NULL, 0, NULL, 0 } };
  int next_option;
  ostringstream sset;
  do {
    next_option = getopt_long (argc, argv, short_options, long_options, NULL);
    switch (next_option) {
    case 'h':
      print_usage();
    case 'a':
      as = strtod(optarg, NULL);
      break;
    case 'n':
      nf = int(strtod(optarg, NULL));
      break;
    case 'F':
      muFrat = strtod(optarg, NULL);
      break;
    case 'L':
      useLLp = true;
      break;
    case '?':
      print_usage();
    case -1: break;
    default: abort();
    }
  }
  while (next_option != -1);
  return;
}



int main (int argc, char* argv[]) {

  struct timeval t0, t1;
  gettimeofday(&t0,NULL);

  bool useLLp = false;
  int nf0 = -1;
  double as0 = -1;
  double muFrat = 1;
  read_arguments(argc, argv, as0, nf0, muFrat, useLLp);

  HELL::SetLLpMode(useLLp);
  HELL::LogOrder ord = HELL::NLL;

  // m values
  const int Nmass = 11;
  double mass[Nmass] = { 1.20, 1.27, 1.35, 1.48, 1.60, 1.80,   // mc
			 4.10, 4.35, 4.55, 4.75, 4.95 }; // mb
  //                     4.10, 4.30, 4.55, 4.80 }; // mb

  // x grid
  int Np1 = 300, Np2 = 20-1;
  double x;
  double x_min = 1e-9;
  double x_mid = 0.1;
  double x_max = 0.99+0.01;

  for(int nf=3; nf<=5; nf++) {
    if(nf0>0 && nf!=nf0) continue;
    HELL::HELLnf sxD(nf, ord, "./data/");
    sxD.SetLargexDamping();
    sxD.SetMomentumConservation(); // has no effect
    sxD.SetEvaluationMethod(1);
    sxD.SetFactorizationScale(muFrat);
    vector<double> as1;
    sxD.GetAvailableAlphas(as1);
    // alphas grid
    //double as_min = as1[0];
    //double as_max = as1[as1.size()-1];
    //if(as2[0]>as_min) as_min = as2[0];
    //if(as2[as2.size()-1]<as_max) as_max = as2[as2.size()-1];
    //int Nas = (as_max-as_min)*1000 +1;
    //Nas = (as_max-as_min)*100 +1; // smaller set; comment out for a (10 times) larger set
    //vector<double> asvals;
    //asvals.push_back(as_min);
    //while(asvals[asvals.size()-1]<as_max) {
      //asvals.push_back(asvals[i]+0.001);
      //asvals.push_back(asvals[asvals.size()-1]+0.01);
    //}
    //
    //for(int a=0; a<Nas; a++) {
    //double as = as_min + a*(as_max-as_min)/(Nas-1.);
    for(unsigned int a=0; a<as1.size(); a++) {
      double as = as1[a];
      //cout << as0 << "   " << as << "   " << as-as0 << endl;
      if(as0>0 && fabs(as-as0)>1e-10) continue;
      as = int(1000*as+0.1)/1000.;
      //cout << setprecision(10) << 1000*as << "  " << sas(as) << endl;
      double Q = Qofalphas(as,nf);
      //cout << Q << endl;
      bool muFneq1 = (fabs(muFrat-1) > 1e-5);
      ostringstream muFs;
      muFs << "_muF" << 1000*muFrat;
      ostringstream filename;
      filename << "./output_for_HELLx/NLL" << (useLLp ? "-LLp" : "") << "/xtableCm" << (muFneq1 ? muFs.str() : "") << "_NLL_nf" << nf << "_alphas" << sas(as) << ".table";
      ofstream ofile(filename.str().c_str());
      ofile << VERSIONx << endl;
      ofile
	<< setw(15) << Np1
	<< setw(15) << Np2
	<< setw(15) << x_min
	<< setw(15) << x_mid
	<< setw(15) << x_max
	<< endl;
      //
      ofile << Nmass << "    " << Q << endl;
      for(int m=0; m<Nmass; m++) {
	double mQ = mass[m]/Q;
	ofile << mQ << endl;
	for(int i=0; i<Np1+Np2; i++) {
	  if(i<Np1) x = x_min * exp(i/(Np1-1.)*log(x_mid/x_min));
	  else      x = x_mid + (i-Np1+1)*(x_max-x_mid)/(Np2-0.);
	  //cout << x << endl;
	  double dKhg    = x*sxD.deltaKhg   (as, x, mQ);
	  double dMC2g   = x*sxD.deltaMC2g  (as, x, mQ);
	  double dMCLg   = x*sxD.deltaMCLg  (as, x, mQ);
	  double dMC2axg = x*sxD.deltaMC2axg(as, x, mQ);
	  double dMCLaxg = x*sxD.deltaMCLaxg(as, x, mQ);
	  double dMC2CCg = x*sxD.deltaMC2CCg(as, x, mQ);
	  double dMCLCCg = x*sxD.deltaMCLCCg(as, x, mQ);
	  double dMC3CCg = x*sxD.deltaMC3CCg(as, x, mQ);
	  ofile
	    //<< setw(15) << x
	    << setw(15) << dKhg
	    << setw(15) << dMC2g
	    << setw(15) << dMCLg
	    << setw(15) << dMC2axg
	    << setw(15) << dMCLaxg
	    << setw(15) << dMC2CCg
	    << setw(15) << dMCLCCg
	    << setw(15) << dMC3CCg
	    << endl;
	}
      }
      cout << ".";
      cout.flush();
      //
      ofile.close();
    }
    cout << endl;
  }


  // Finish time
  gettimeofday(&t1,NULL);
  double t=t1.tv_sec-t0.tv_sec+(t1.tv_usec-t0.tv_usec)*0.000001;
  cout << "Total time: " << t << "s" << endl;

  return 0;

}
