#include <vector>
#include <integration.hh>
#include <gsl/gsl_sf.h>
#include <gsl/gsl_integration.h>

namespace HELL {

  // composite simpson rule
  Real SimpsonIntegration::simpson_rule_(Real fct_vals[],
                                         Real a, Real b, unsigned n)
  {
    Real h = (b-a)/n;
    Real s = fct_vals[0]+fct_vals[n];
    /*
    for (unsigned i=1; i<n; i+=2)
      s += 4*fct_vals[i];
    for (unsigned i=2; i<n-1; i+=2)
      s += 2*fct_vals[i];
    */
    for (unsigned i=1; i<n; i++) {
      if(i%2==0) s += 2*fct_vals[i];
      else       s += 4*fct_vals[i];
    }
    return s*h/3;
  }

  Real SimpsonIntegration::integrate(Real fct_vals[],
                                     Real a, Real b, unsigned n,
                                     Real * err)
  {
    if (n%2) {
      std::cerr << "ERROR in hell::SimpsonIntegration::integrate :\n"
                << "number n of intervals needs to be even" << std::endl;
      std::terminate();
    }
    
    Real res = simpson_rule_(fct_vals,a,b,n);

    if (err) {

      // the error is estimated by taking the difference between the
      // result and a less accurate calculation which uses only half
      // of the points
      
      if (n%4) {
        std::cerr << "ERROR in hell::SimpsonIntegration::integrate :\n"
                  << "for error estimation, number n of intervals "
                  << "needs to be multiple of 4" << std::endl;
        std::terminate();
      }
      
      std::vector<Real> half_fct_vals(n/2+1);

      for (unsigned i=0; i<half_fct_vals.size(); ++i)
        half_fct_vals[i] = fct_vals[2*i];

      Real res2 = simpson_rule_(&half_fct_vals[0],a,b,n/2);

      *err = std::abs(res-res2);
    }

    return res;
  }

  double integration_gauss(double f(double,void*), double xmin, double xmax, double prec, double *error, void *par, int *status){
  double result, err;
  gsl_function F;
  gsl_integration_workspace * w = gsl_integration_workspace_alloc (1000);
  F.function = f;
  F.params=par;
  //int s = gsl_integration_qag  (&F, xmin, xmax, 0, prec, 1000, 2, w, &result, &err);
  //int s = gsl_integration_qag  (&F, xmin, xmax, 0, prec, 1000, 6, w, &result, &err);
  int s = gsl_integration_qags (&F, xmin, xmax, 0, prec, 1000,    w, &result, &err);
  gsl_integration_workspace_free (w);
  if(status) *status = s;
  if(error) *error = err;
  return result;
}

double integration_gauss_inf(double f(double,void*), double xmin, double prec, double *error, void *par, int *status){
  double result, err;
  gsl_function F;
  gsl_integration_workspace * w = gsl_integration_workspace_alloc (100000);
  F.function = f;
  F.params=par;
  int s = gsl_integration_qagiu (&F, xmin, 0, prec, 100000, w, &result, &err);
  gsl_integration_workspace_free (w);
  if(status) *status = s;
  if(error) *error = err;
  return result;
}
 
} // namespace hell
